//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "formTAPIMain.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"

TForm1 *Form1;

/****************************************************
Purpose:   This function reads a value from the registry on the specified PC

Arguments: String sPCName  = Name of the PC to read registry data from
           HKEY hKeyType   = The name of the registry key to read
           String sKeyName = The name of the registry branch to read
           String sName    = The Data Value item to read
           int vType       = The type of data to read (varInteger, varString, etc.)

Returns    Variant = the value read

****************************************************/
Variant ReadFromRegistry(String sPCName, HKEY hKeyType, String sKeyName, String sName, int vType)
{
  Variant vData;
  vData = vData.AsType(varNull);

  try
  {
    if((sKeyName != ""))
    {
      std::auto_ptr<TRegistry> myReg(new TRegistry());

      if(sPCName != "")
      {
        if (myReg->RegistryConnect("\\\\" + sPCName) == false)
        {
          return(NULL);
        }
      }

      myReg->RootKey = hKeyType;

      if (myReg->OpenKey(sKeyName,false))
      {
        if(myReg->ValueExists(sName) == true || sName == "")
        {
          switch (vType)
          {
            case varEmpty:

              break;
            case varNull:

              break;
            case varSmallint:
              vData = myReg->ReadInteger(sName);
              break;
            case varInteger:
              vData = myReg->ReadInteger(sName);
              break;
            case varSingle:
              vData = myReg->ReadFloat(sName);
              break;
            case varDouble:
              vData = myReg->ReadFloat(sName);
              break;
            case varCurrency:
              vData = myReg->ReadCurrency(sName);
              break;
            case varDate:
              vData = myReg->ReadDate(sName);
              break;
            case varOleStr:
              vData = myReg->ReadString(sName);
              break;
            case varDispatch:

              break;
            case varError:

              break;
            case varBoolean:
              vData = myReg->ReadBool(sName);
              break;
            case varVariant:
              //myReg->ReadBinaryData(sName,vData,sizeof(vData));
              break;
            case varUnknown:

              break;
            case varByte:
              vData = myReg->ReadString(sName);
              break;
            case varString:
              vData = myReg->ReadString(sName);
              break;
            case varTypeMask:

              break;
            case varArray:

              break;
            case varByRef:

              break;
            default:

              break;
          }
        }
      }


      myReg->CloseKey();
      //delete myReg;
    }
  }
  catch(Exception &E)
  {
    AnsiString asMsg = "Internal Error in [PACKAGE method ReadFromRegistry()] msg: " + E.Message;
    Application->MessageBox(asMsg.c_str(),"Problem Detected",MB_OK);
  }

  return(vData);
}

/****************************************************
Purpose:   This function writes a value to the registry on the specified PC

Arguments: String sPCName  = Name of the PC to read registry data from
           HKEY hKeyType   = The name of the registry key to read
           String sKeyName = The name of the registry branch to read
           String sName    = The Data Value item to read
           Variant vData   = The Data item to write

Returns    bool = success = true

****************************************************/
bool WriteToRegistry(String sPCName, HKEY hKeyType, String sKeyName, String sName, Variant vData)
{
  bool bSuccess = false;

  try
  {
    if((sKeyName != "") && (sName != ""))
    {
      std::auto_ptr<TRegistry> myReg(new TRegistry());

      if(sPCName != "")
      {
        if (myReg->RegistryConnect("\\\\" + sPCName) == false)
        {
          return(bSuccess);
        }
      }

      myReg->RootKey = hKeyType;

      if (myReg->OpenKey(sKeyName,True))
      {
        switch (vData.Type())
        {
          case varEmpty:

            break;
          case varNull:

            break;
          case varSmallint:
            myReg->WriteInteger(sName,vData);
            bSuccess = true;
            break;
          case varInteger:
            myReg->WriteInteger(sName,vData);
            bSuccess = true;
            break;
          case varSingle:
            myReg->WriteFloat(sName,vData);
            bSuccess = true;
            break;
          case varDouble:
            myReg->WriteFloat(sName,vData);
            bSuccess = true;
            break;
          case varCurrency:
            myReg->WriteCurrency(sName,vData);
            bSuccess = true;
            break;
          case varDate:
            myReg->WriteDate(sName,vData);
            bSuccess = true;
            break;
          case varOleStr:
            myReg->WriteString(sName,vData);
            bSuccess = true;
            break;
          case varDispatch:

            break;
          case varError:

            break;
          case varBoolean:
            myReg->WriteBool(sName,vData);
            bSuccess = true;
            break;
          case varVariant:
            //myReg->WriteBinaryData(sName,vData,sizeof(vData));
            bSuccess = true;
            break;
          case varUnknown:

            break;
          case varByte:
            myReg->WriteString(sName,vData);
            bSuccess = true;
            break;
          case varString:
            myReg->WriteString(sName,vData);
            bSuccess = true;
            break;
          case varTypeMask:

            break;
          case varArray:

            break;
          case varByRef:

            break;
          default:

            break;
        }

        RegFlushKey(HKEY_LOCAL_MACHINE);
      }


      myReg->CloseKey();
    }
  }
  catch(Exception &E)
  {
    AnsiString asMsg = "Internal Error in [PACKAGE method WriteToRegistry()] msg: " + E.Message;
    Application->MessageBox(asMsg.c_str(),"Problem Detected",MB_OK);

  }

  return(bSuccess);
}

//---------------------------------------------------------------------------
__fastcall TForm1::TForm1(TComponent* Owner)
  : TForm(Owner)
{
  edtTestWav->Text = "";
  if(ReadFromRegistry("", HKEY_CURRENT_USER, "SOFTWARE\\VSOFT\\TAPITest", "WavFile" , varString).IsNull() == false)
  {
    edtTestWav->Text = ReadFromRegistry("", HKEY_CURRENT_USER, "SOFTWARE\\VSOFT\\TAPITest", "WavFile" , varString);
  }

  asCallPhoneNum_m = "";
  if(ReadFromRegistry("", HKEY_CURRENT_USER, "SOFTWARE\\VSOFT\\TAPITest", "CallPhoneNumber" , varString).IsNull() == false)
  {
    asCallPhoneNum_m = ReadFromRegistry("", HKEY_CURRENT_USER, "SOFTWARE\\VSOFT\\TAPITest", "CallPhoneNumber" , varString);
  }

  edtMaxRecordTime->Text = "300";
  if(ReadFromRegistry("", HKEY_CURRENT_USER, "SOFTWARE\\VSOFT\\TAPITest", "MaxRecordSecs" , varInteger).IsNull() == false)
  {
    edtMaxRecordTime->Text = (int)ReadFromRegistry("", HKEY_CURRENT_USER, "SOFTWARE\\VSOFT\\TAPITest", "MaxRecordSecs" , varInteger);
  }

}

//---------------------------------------------------------------------------
void __fastcall TForm1::bbtnOpenTAPIClick(TObject *Sender)
{
  DWORD dwTAPIDeviceID = -1;
  LPLINEDEVCAPS lpDevCaps = NULL;

  try
  {
    lpDevCaps = vsTAPI->GetDeviceCapabilities(dwTAPIDeviceID);
  }
  catch(...)
  {
    lpDevCaps = vsTAPI->GetDeviceCapabilities(dwTAPIDeviceID,true);
  }

  if(lpDevCaps)
  {
    if((lmmAUTOMATEDVOICE & lpDevCaps->dwMediaModes))
    {
      vsTAPI->MediaModes = vsTAPI->MediaModes | lmmAUTOMATEDVOICE;
    }

    if((lmmAUTOMATEDVOICE & lpDevCaps->dwMediaModes))
    {
      vsTAPI->MediaModes = vsTAPI->MediaModes | lmmAUTOMATEDVOICE;

      if((lmmINTERACTIVEVOICE & lpDevCaps->dwMediaModes))
      {
        vsTAPI->MediaModes = vsTAPI->MediaModes | lmmINTERACTIVEVOICE;
      }
    }
    else if((lmmDATAMODEM & lpDevCaps->dwMediaModes))
    {
      vsTAPI->MediaModes = vsTAPI->MediaModes | lmmDATAMODEM;
    }

    try
    {
      vsTAPI->Privileges = lcpMONITOR_AND_OWNER;
      if(vsTAPI->OpenDevice(dwTAPIDeviceID,false) == false)
      {
        //ShowMessage("Error Opening TAPI Device!");
        throw EAbort("");
      }
    }
    catch(...)
    {
      vsTAPI->Privileges = lcpMONITOR_AND_OWNER;
      if(vsTAPI->OpenDevice(dwTAPIDeviceID,true) == false)
      {
        ShowMessage("Error Opening TAPI Device! Detected TAPI Version: " + vsTAPI->TAPIVersionString);
      }
    }
  }  
}

//---------------------------------------------------------------------------
void __fastcall TForm1::vsTAPITapiError(AnsiString asErrorDesc,
      int iErrorSection, int iFunctionID)
{
  AnsiString asMsg = "Internal TAPI error in section [" + IntToStr(iErrorSection) + "] function [" + IntToStr(iFunctionID) + "] msg: " + asErrorDesc;

  memoTAPI->Lines->Add(asMsg);

  ShowMessage(asMsg);
}

//---------------------------------------------------------------------------
void __fastcall TForm1::vsTAPICallInfoEvent(LINEMESSAGE *cbData,
      LINECALLINFO *pLineCallInfo)
{
  int iMsgID    = cbData->dwMessageID;
  int iDwParam1 = cbData->dwParam1;
  int iDwParam2 = cbData->dwParam2;

  AnsiString asMsg = "N/A";
  AnsiString asCIDMsg = "N/A";
  switch(iMsgID)
  {
    case LINE_ADDRESSSTATE:
      asMsg = "The LINE_ADDRESSSTATE message is sent when the status of an address changes on a line that is currently open by the application. The application can invoke lineGetAddressStatus to determine the current status of the address.";
      asCIDMsg = "TAPI Detected an event: [LINE_ADDRESSSTATE] Param1: [" + IntToStr(iDwParam1) + "] Param2: [" + IntToStr(iDwParam2) + "] Text: [" + asMsg + "]";
      break;

    case LINE_CALLINFO:
      switch(iDwParam1)
      {
        case LINECALLINFOSTATE_OTHER:
          asMsg = "Information items other than those listed below have changed. The application should check the current call information to determine which items have changed.";
          asCIDMsg = "TAPI Detected an event: [LINECALLINFOSTATE_OTHER] Param1: [" + IntToStr(iDwParam1) + "] Param2: [" + IntToStr(iDwParam2) + "] Text: [" + asMsg + "]";
          break;

        case LINECALLINFOSTATE_DEVSPECIFIC:
          asMsg = "The device-specific field of the call-information record has changed.";
          asCIDMsg = "TAPI Detected an event: [LINECALLINFOSTATE_DEVSPECIFIC] Param1: [" + IntToStr(iDwParam1) + "] Param2: [" + IntToStr(iDwParam2) + "] Text: [" + asMsg + "]";
          break;

        case LINECALLINFOSTATE_BEARERMODE:
          asMsg = "The bearer mode field of the call-information record has changed.";
          asCIDMsg = "TAPI Detected an event: [LINECALLINFOSTATE_BEARERMODE] Param1: [" + IntToStr(iDwParam1) + "] Param2: [" + IntToStr(iDwParam2) + "] Text: [" + asMsg + "]";
          break;

        case LINECALLINFOSTATE_RATE:
          asMsg = "The rate field of the call-information record has changed.";
          asCIDMsg = "TAPI Detected an event: [LINECALLINFOSTATE_RATE] Param1: [" + IntToStr(iDwParam1) + "] Param2: [" + IntToStr(iDwParam2) + "] Text: [" + asMsg + "]";
          break;

        case LINECALLINFOSTATE_MEDIAMODE:
          asMsg = "The media mode field of the call-information record has changed.";
          asCIDMsg = "TAPI Detected an event: [LINECALLINFOSTATE_MEDIAMODE] Param1: [" + IntToStr(iDwParam1) + "] Param2: [" + IntToStr(iDwParam2) + "] Text: [" + asMsg + "]";
          break;

        case LINECALLINFOSTATE_APPSPECIFIC:
          asMsg = "The application-specific field of the call-information record has changed.";
          asCIDMsg = "TAPI Detected an event: [LINECALLINFOSTATE_APPSPECIFIC] Param1: [" + IntToStr(iDwParam1) + "] Param2: [" + IntToStr(iDwParam2) + "] Text: [" + asMsg + "]";
          break;

        case LINECALLINFOSTATE_CALLID:
          asMsg = "The call ID field of the call-information record has changed.";
          asCIDMsg = "TAPI Detected an event: [LINECALLINFOSTATE_CALLID] Param1: [" + IntToStr(iDwParam1) + "] Param2: [" + IntToStr(iDwParam2) + "] Text: [" + asMsg + "]";
          break;

        case LINECALLINFOSTATE_RELATEDCALLID:
          asMsg = "The related call ID field of the call-information record has changed. ";
          asCIDMsg = "TAPI Detected an event: [LINECALLINFOSTATE_RELATEDCALLID] Param1: [" + IntToStr(iDwParam1) + "] Param2: [" + IntToStr(iDwParam2) + "] Text: [" + asMsg + "]";
          break;

        case LINECALLINFOSTATE_ORIGIN:
          asMsg = "The origin field of the call-information record has changed. ";
          asCIDMsg = "TAPI Detected an event: [LINECALLINFOSTATE_ORIGIN] Param1: [" + IntToStr(iDwParam1) + "] Param2: [" + IntToStr(iDwParam2) + "] Text: [" + asMsg + "]";
          break;

        case LINECALLINFOSTATE_REASON:
          asMsg = "The reason field of the call-information record has changed. ";
          asCIDMsg = "TAPI Detected an event: [LINECALLINFOSTATE_REASON] Param1: [" + IntToStr(iDwParam1) + "] Param2: [" + IntToStr(iDwParam2) + "] Text: [" + asMsg + "]";
          break;

        case LINECALLINFOSTATE_COMPLETIONID:
          asMsg = "The completion ID field of the call-information record has changed. ";
          asCIDMsg = "TAPI Detected an event: [LINECALLINFOSTATE_COMPLETIONID] Param1: [" + IntToStr(iDwParam1) + "] Param2: [" + IntToStr(iDwParam2) + "] Text: [" + asMsg + "]";
          break;

        case LINECALLINFOSTATE_NUMOWNERINCR:
          asMsg = "The number of owner fields in the call-information record was increased.";
          asCIDMsg = "TAPI Detected an event: [LINECALLINFOSTATE_NUMOWNERINCR] Param1: [" + IntToStr(iDwParam1) + "] Param2: [" + IntToStr(iDwParam2) + "] Text: [" + asMsg + "]";
          break;

        case LINECALLINFOSTATE_NUMOWNERDECR:
          asMsg = "The number of owner fields in the call-information record was decreased.";
          asCIDMsg = "TAPI Detected an event: [LINECALLINFOSTATE_NUMOWNERDECR] Param1: [" + IntToStr(iDwParam1) + "] Param2: [" + IntToStr(iDwParam2) + "] Text: [" + asMsg + "]";
          break;

        case LINECALLINFOSTATE_NUMMONITORS:
          asMsg = "The number of monitors fields in the call-information record has changed.";
          asCIDMsg = "TAPI Detected an event: [LINECALLINFOSTATE_NUMMONITORS] Param1: [" + IntToStr(iDwParam1) + "] Param2: [" + IntToStr(iDwParam2) + "] Text: [" + asMsg + "]";
          break;

        case LINECALLINFOSTATE_TRUNK:
          asMsg = "The trunk field of the call information record has changed.";
          asCIDMsg = "TAPI Detected an event: [LINECALLINFOSTATE_TRUNK] Param1: [" + IntToStr(iDwParam1) + "] Param2: [" + IntToStr(iDwParam2) + "] Text: [" + asMsg + "]";
          break;

        case LINECALLINFOSTATE_CALLERID:
          asMsg = "One of the callerID-related fields of the call information record has changed.";
          asCIDMsg = "TAPI Detected an event: [LINECALLINFOSTATE_CALLERID] Param1: [" + IntToStr(iDwParam1) + "] Param2: [" + IntToStr(iDwParam2) + "] dwCallerIDFlags: [" + IntToStr(pLineCallInfo->dwCallerIDFlags) + "] Text: [" + asMsg + "]";
          break;

        case LINECALLINFOSTATE_CALLEDID:
          asMsg = "One of the calledID-related fields of the call information record has changed.";
          asCIDMsg = "TAPI Detected an event: [LINECALLINFOSTATE_CALLEDID] Param1: [" + IntToStr(iDwParam1) + "] Param2: [" + IntToStr(iDwParam2) + "] Text: [" + asMsg + "]";
          break;

        case LINECALLINFOSTATE_CONNECTEDID:
          asMsg = "One of the connectedID-related fields of the call information record has changed.";
          asCIDMsg = "TAPI Detected an event: [LINECALLINFOSTATE_CONNECTEDID] Param1: [" + IntToStr(iDwParam1) + "] Param2: [" + IntToStr(iDwParam2) + "] Text: [" + asMsg + "]";
          break;

        case LINECALLINFOSTATE_REDIRECTIONID:
          asMsg = "One of the redirectionID-related fields of the call information record has changed.";
          asCIDMsg = "TAPI Detected an event: [LINECALLINFOSTATE_REDIRECTIONID] Param1: [" + IntToStr(iDwParam1) + "] Param2: [" + IntToStr(iDwParam2) + "] Text: [" + asMsg + "]";
          break;

        case LINECALLINFOSTATE_REDIRECTINGID:
          asMsg = "One of the redirectingID-related fields of the call information record has changed.";
          asCIDMsg = "TAPI Detected an event: [LINECALLINFOSTATE_REDIRECTINGID] Param1: [" + IntToStr(iDwParam1) + "] Param2: [" + IntToStr(iDwParam2) + "] Text: [" + asMsg + "]";
          break;

        case LINECALLINFOSTATE_DISPLAY:
          asMsg = "The display field of the call information record has changed.";
          asCIDMsg = "TAPI Detected an event: [LINECALLINFOSTATE_DISPLAY] Param1: [" + IntToStr(iDwParam1) + "] Param2: [" + IntToStr(iDwParam2) + "] Text: [" + asMsg + "]";
          break;

        case LINECALLINFOSTATE_USERUSERINFO:
          asMsg = "The user-to-user information of the call information record has changed.";
          asCIDMsg = "TAPI Detected an event: [LINECALLINFOSTATE_USERUSERINFO] Param1: [" + IntToStr(iDwParam1) + "] Param2: [" + IntToStr(iDwParam2) + "] Text: [" + asMsg + "]";
          break;

        case LINECALLINFOSTATE_HIGHLEVELCOMP:
          asMsg = "The high-level compatibility field of the call information record has changed.";
          asCIDMsg = "TAPI Detected an event: [LINECALLINFOSTATE_HIGHLEVELCOMP] Param1: [" + IntToStr(iDwParam1) + "] Param2: [" + IntToStr(iDwParam2) + "] Text: [" + asMsg + "]";
          break;

        case LINECALLINFOSTATE_LOWLEVELCOMP:
          asMsg = "The low-level compatibility field of the call information record has changed.";
          asCIDMsg = "TAPI Detected an event: [LINECALLINFOSTATE_LOWLEVELCOMP] Param1: [" + IntToStr(iDwParam1) + "] Param2: [" + IntToStr(iDwParam2) + "] Text: [" + asMsg + "]";
          break;

        case LINECALLINFOSTATE_CHARGINGINFO:
          asMsg = "The charging information of the call information record has changed.";
          asCIDMsg = "TAPI Detected an event: [LINECALLINFOSTATE_CHARGINGINFO] Param1: [" + IntToStr(iDwParam1) + "] Param2: [" + IntToStr(iDwParam2) + "] Text: [" + asMsg + "]";
          break;

        case LINECALLINFOSTATE_TERMINAL:
          asMsg = "The terminal mode information of the call information record has changed.";
          asCIDMsg = "TAPI Detected an event: [LINECALLINFOSTATE_TERMINAL] Param1: [" + IntToStr(iDwParam1) + "] Param2: [" + IntToStr(iDwParam2) + "] Text: [" + asMsg + "]";
          break;

        case LINECALLINFOSTATE_DIALPARAMS:
          asMsg = "The dial parameters of the call information record has changed.";
          asCIDMsg = "TAPI Detected an event: [LINECALLINFOSTATE_DIALPARAMS] Param1: [" + IntToStr(iDwParam1) + "] Param2: [" + IntToStr(iDwParam2) + "] Text: [" + asMsg + "]";
          break;

        case LINECALLINFOSTATE_MONITORMODES:
          asMsg = "One or more of the digit, tone, or media monitoring fields in the call information record has changed.";
          asCIDMsg = "TAPI Detected an event: [LINECALLINFOSTATE_MONITORMODES] Param1: [" + IntToStr(iDwParam1) + "] Param2: [" + IntToStr(iDwParam2) + "] Text: [" + asMsg + "]";
          break;

        default:
          asMsg = "UNKNOWN LINECALLINFOSTATE TAPI EVENT";
          asCIDMsg = "TAPI Detected an event with ID: [" + IntToStr(iMsgID) + "] Param1: [" + IntToStr(iDwParam1) + "] Text: [" + asMsg + "]";
          break;
      }

      break;

    case LINE_CALLSTATE:
      switch(iDwParam1)
      {
        case LINECALLSTATE_IDLE:
          asMsg = "The call is idle--no call exists.";
          asCIDMsg = "TAPI Detected an event: [LINECALLSTATE_IDLE] Param1: [" + IntToStr(iDwParam1) + "] Text: [" + asMsg + "]";
          break;
        case LINECALLSTATE_OFFERING:
          asMsg = "The call is being offered to the station, signaling the arrival of a new call. In some environments, a call in the offering state does not automatically alert the user because alerting is done by the switch instructing the line to ring. It does not affect any call states.";
          if(iDwParam2 == LINEOFFERINGMODE_ACTIVE)
          {
            asCIDMsg = "TAPI Detected an event: [LINECALLSTATE_OFFERING] Param1: [LINEOFFERINGMODE_ACTIVE] Text: [" + asMsg + "]";
          }
          else if(iDwParam2 == LINEOFFERINGMODE_INACTIVE)
          {
            asCIDMsg = "TAPI Detected an event: [LINECALLSTATE_OFFERING] Param1: [LINEOFFERINGMODE_INACTIVE] Text: [" + asMsg + "]";
          }
          else
          {
            asCIDMsg = "TAPI Detected an event: [LINECALLSTATE_OFFERING] Param1: [" + IntToStr(iDwParam1) + "] Text: [" + asMsg + "]";
          }

          break;
        case LINECALLSTATE_ACCEPTED:
          asMsg = "The call was offering and has been accepted. This indicates to other (monitoring) applications that the current owner application has claimed responsibility for answering the call. In ISDN, this also initiates alerting to both parties.";
          asCIDMsg = "TAPI Detected an event: [LINECALLSTATE_ACCEPTED] Param1: [" + IntToStr(iDwParam1) + "] Text: [" + asMsg + "]";
          break;
        case LINECALLSTATE_DIALTONE:
          asMsg = "The call is receiving a dial tone from the switch, which means that the switch is ready to receive a dialed number.";
          if(iDwParam2 == LINEDIALTONEMODE_NORMAL)
          {
            asCIDMsg = "TAPI Detected an event: [LINECALLSTATE_DIALTONE] Param1: [LINEDIALTONEMODE_NORMAL] Text: [" + asMsg + "]";
          }
          else if(iDwParam2 == LINEDIALTONEMODE_SPECIAL)
          {
            asCIDMsg = "TAPI Detected an event: [LINECALLSTATE_DIALTONE] Param1: [LINEDIALTONEMODE_SPECIAL] Text: [" + asMsg + "]";
          }
          else if(iDwParam2 == LINEDIALTONEMODE_INTERNAL)
          {
            asCIDMsg = "TAPI Detected an event: [LINECALLSTATE_DIALTONE] Param1: [LINEDIALTONEMODE_INTERNAL] Text: [" + asMsg + "]";
          }
          else if(iDwParam2 == LINEDIALTONEMODE_EXTERNAL)
          {
            asCIDMsg = "TAPI Detected an event: [LINECALLSTATE_DIALTONE] Param1: [LINEDIALTONEMODE_EXTERNAL] Text: [" + asMsg + "]";
          }
          else if(iDwParam2 == LINEDIALTONEMODE_UNKNOWN)
          {
            asCIDMsg = "TAPI Detected an event: [LINECALLSTATE_DIALTONE] Param1: [LINEDIALTONEMODE_UNKNOWN] Text: [" + asMsg + "]";
          }
          else if(iDwParam2 == LINEDIALTONEMODE_UNAVAIL)
          {
            asCIDMsg = "TAPI Detected an event: [LINECALLSTATE_DIALTONE] Param1: [LINEDIALTONEMODE_UNAVAIL] Text: [" + asMsg + "]";
          }
          else
          {
            asCIDMsg = "TAPI Detected an event: [LINECALLSTATE_DIALTONE] Param1: [" + IntToStr(iDwParam1) + "] Text: [" + asMsg + "]";
          }

          break;
        case LINECALLSTATE_DIALING:
          asMsg = "Destination address information (a phone number) is being sent to the switch over the call. Note that lineGenerateDigits does not place the line into the dialing state.";
          asCIDMsg = "TAPI Detected an event: [LINECALLSTATE_DIALING] Param1: [" + IntToStr(iDwParam1) + "] Text: [" + asMsg + "]";
          break;
        case LINECALLSTATE_RINGBACK:
          asMsg = "The call is receiving ringback from the called address. Ringback indicates that the other station has been reached and is being alerted.";
          asCIDMsg = "TAPI Detected an event: [LINECALLSTATE_RINGBACK] Param1: [" + IntToStr(iDwParam1) + "] Text: [" + asMsg + "]";
          break;
        case LINECALLSTATE_BUSY:
          asMsg = "The call is receiving a busy tone. A busy tone indicates that the call cannot be completed--either a circuit (trunk) or the remote party's station are in use.";
          if(iDwParam2 == LINEBUSYMODE_STATION)
          {
            asCIDMsg = "TAPI Detected an event: [LINECALLSTATE_BUSY] Param1: [LINEBUSYMODE_STATION] Text: [" + asMsg + "]";
          }
          else if(iDwParam2 == LINEBUSYMODE_TRUNK)
          {
            asCIDMsg = "TAPI Detected an event: [LINECALLSTATE_BUSY] Param1: [LINEBUSYMODE_TRUNK] Text: [" + asMsg + "]";
          }
          else if(iDwParam2 == LINEBUSYMODE_UNKNOWN)
          {
            asCIDMsg = "TAPI Detected an event: [LINECALLSTATE_BUSY] Param1: [LINEBUSYMODE_UNKNOWN] Text: [" + asMsg + "]";
          }
          else if(iDwParam2 == LINEBUSYMODE_UNAVAIL)
          {
            asCIDMsg = "TAPI Detected an event: [LINECALLSTATE_BUSY] Param1: [LINEBUSYMODE_UNAVAIL] Text: [" + asMsg + "]";
          }
          else
          {
            asCIDMsg = "TAPI Detected an event: [LINECALLSTATE_BUSY] Param1: [" + IntToStr(iDwParam1) + "] Text: [" + asMsg + "]";
          }

          break;
        case LINECALLSTATE_SPECIALINFO:
          asMsg = "Special information is sent by the network. Special information is typically sent when the destination cannot be reached.";
          asCIDMsg = "TAPI Detected an event: [LINECALLSTATE_SPECIALINFO] Param1: [" + IntToStr(iDwParam1) + "] Text: [" + asMsg + "]";
          break;
        case LINECALLSTATE_CONNECTED:
          asMsg = "The call has been established and the connection is made. Information is able to flow over the call between the originating address and the destination address.";
          if(iDwParam2 == LINECONNECTEDMODE_ACTIVE)
          {
            asCIDMsg = "TAPI Detected an event: [LINECALLSTATE_CONNECTED] Param1: [LINECONNECTEDMODE_ACTIVE] Text: [" + asMsg + "]";
          }
          else if(iDwParam2 == LINECONNECTEDMODE_INACTIVE)
          {
            asCIDMsg = "TAPI Detected an event: [LINECALLSTATE_CONNECTED] Param1: [LINECONNECTEDMODE_INACTIVE] Text: [" + asMsg + "]";
          }
          else
          {
            asCIDMsg = "TAPI Detected an event: [LINECALLSTATE_CONNECTED] Param1: [" + IntToStr(iDwParam1) + "] Text: [" + asMsg + "]";
          }

          break;
        case LINECALLSTATE_PROCEEDING:
          asMsg = "Dialing has completed and the call is proceeding through the switch or telephone network.";
          asCIDMsg = "TAPI Detected an event: [LINECALLSTATE_PROCEEDING] Param1: [" + IntToStr(iDwParam1) + "] Text: [" + asMsg + "]";
          break;
        case LINECALLSTATE_ONHOLD:
          asMsg = "The call is on hold by the switch.";
          asCIDMsg = "TAPI Detected an event: [LINECALLSTATE_ONHOLD] Param1: [" + IntToStr(iDwParam1) + "] Text: [" + asMsg + "]";
          break;
        case LINECALLSTATE_CONFERENCED:
          asMsg = "The call is currently a member of a multi-party conference call.";
          asCIDMsg = "TAPI Detected an event: [LINECALLSTATE_CONFERENCED] Param1: [" + IntToStr(iDwParam1) + "] Text: [" + asMsg + "]";
          break;
        case LINECALLSTATE_ONHOLDPENDCONF:
          asMsg = "The call is currently on hold while it is being added to a conference.";
          asCIDMsg = "TAPI Detected an event: [LINECALLSTATE_ONHOLDPENDCONF] Param1: [" + IntToStr(iDwParam1) + "] Text: [" + asMsg + "]";
          break;
        case LINECALLSTATE_ONHOLDPENDTRANSFER:
          asMsg = "The call is currently on hold awaiting transfer to another number.";
          asCIDMsg = "TAPI Detected an event: [LINECALLSTATE_ONHOLDPENDTRANSFER] Param1: [" + IntToStr(iDwParam1) + "] Text: [" + asMsg + "]";
          break;
        case LINECALLSTATE_DISCONNECTED:
          asMsg = "The remote party has disconnected from the call.";
          if(iDwParam2 == LINEDISCONNECTMODE_NORMAL)
          {
            asCIDMsg = "TAPI Detected an event: [LINECALLSTATE_DISCONNECTED] Param1: [LINEDISCONNECTMODE_NORMAL] Text: [" + asMsg + "]";
          }
          else if(iDwParam2 == LINEDISCONNECTMODE_UNKNOWN)
          {
            asCIDMsg = "TAPI Detected an event: [LINECALLSTATE_DISCONNECTED] Param1: [LINEDISCONNECTMODE_UNKNOWN] Text: [" + asMsg + "]";
          }
          else if(iDwParam2 == LINEDISCONNECTMODE_REJECT)
          {
            asCIDMsg = "TAPI Detected an event: [LINECALLSTATE_DISCONNECTED] Param1: [LINEDISCONNECTMODE_REJECT] Text: [" + asMsg + "]";
          }
          else if(iDwParam2 == LINEDISCONNECTMODE_PICKUP)
          {
            asCIDMsg = "TAPI Detected an event: [LINECALLSTATE_DISCONNECTED] Param1: [LINEDISCONNECTMODE_PICKUP] Text: [" + asMsg + "]";
          }
          else if(iDwParam2 == LINEDISCONNECTMODE_FORWARDED)
          {
            asCIDMsg = "TAPI Detected an event: [LINECALLSTATE_DISCONNECTED] Param1: [LINEDISCONNECTMODE_FORWARDED] Text: [" + asMsg + "]";
          }
          else if(iDwParam2 == LINEDISCONNECTMODE_BUSY)
          {
            asCIDMsg = "TAPI Detected an event: [LINECALLSTATE_DISCONNECTED] Param1: [LINEDISCONNECTMODE_BUSY] Text: [" + asMsg + "]";
          }
          else if(iDwParam2 == LINEDISCONNECTMODE_NOANSWER)
          {
            asCIDMsg = "TAPI Detected an event: [LINECALLSTATE_DISCONNECTED] Param1: [LINEDISCONNECTMODE_NOANSWER] Text: [" + asMsg + "]";
          }
          else if(iDwParam2 == LINEDISCONNECTMODE_NODIALTONE)
          {
            asCIDMsg = "TAPI Detected an event: [LINECALLSTATE_DISCONNECTED] Param1: [LINEDISCONNECTMODE_NODIALTONE] Text: [" + asMsg + "]";
          }
          else if(iDwParam2 == LINEDISCONNECTMODE_BADADDRESS)
          {
            asCIDMsg = "TAPI Detected an event: [LINECALLSTATE_DISCONNECTED] Param1: [LINEDISCONNECTMODE_BADADDRESS] Text: [" + asMsg + "]";
          }
          else if(iDwParam2 == LINEDISCONNECTMODE_UNREACHABLE)
          {
            asCIDMsg = "TAPI Detected an event: [LINECALLSTATE_DISCONNECTED] Param1: [LINEDISCONNECTMODE_UNREACHABLE] Text: [" + asMsg + "]";
          }
          else if(iDwParam2 == LINEDISCONNECTMODE_CONGESTION)
          {
            asCIDMsg = "TAPI Detected an event: [LINECALLSTATE_DISCONNECTED] Param1: [LINEDISCONNECTMODE_CONGESTION] Text: [" + asMsg + "]";
          }
          else if(iDwParam2 == LINEDISCONNECTMODE_INCOMPATIBLE)
          {
            asCIDMsg = "TAPI Detected an event: [LINECALLSTATE_DISCONNECTED] Param1: [LINEDISCONNECTMODE_INCOMPATIBLE] Text: [" + asMsg + "]";
          }
          else if(iDwParam2 == LINEDISCONNECTMODE_UNAVAIL)
          {
            asCIDMsg = "TAPI Detected an event: [LINECALLSTATE_DISCONNECTED] Param1: [LINEDISCONNECTMODE_UNAVAIL] Text: [" + asMsg + "]";
          }
          else
          {
            asCIDMsg = "TAPI Detected an event: [LINECALLSTATE_DISCONNECTED] Param1: [" + IntToStr(iDwParam1) + "] Text: [" + asMsg + "]";
          }

          break;

        case LINECALLSTATE_UNKNOWN:
          asMsg = "The state of the call is not known. This may be due to limitations of the call-progress detection implementation.";
          asCIDMsg = "TAPI Detected an event: [LINECALLSTATE_UNKNOWN] Param1: [" + IntToStr(iDwParam1) + "] Text: [" + asMsg + "]";
          break;

        default:
          asMsg = "UNKNOWN LINECALLSTATE TAPI EVENT";
          asCIDMsg = "TAPI Detected an event with ID: [" + IntToStr(iMsgID) + "] Param1: [" + IntToStr(iDwParam1) + "] Param2: [" + IntToStr(iDwParam2) + "] Text: [" + asMsg + "]";

          break;
      }

      break;

    case LINE_CLOSE:
      asMsg = "The LINE_CLOSE message is sent when the specified line device has been forcibly closed. The line device handle or any call handles for calls on the line are no longer valid once this message has been sent.";
      asCIDMsg = "TAPI Detected an event: [LINE_CLOSE] Param1: [" + IntToStr(iDwParam1) + "] Param2: [" + IntToStr(iDwParam2) + "] Text: [" + asMsg + "]";
      break;

    case LINE_DEVSPECIFIC:
      asMsg = "The LINE_DEVSPECIFIC message is sent to notify the application about device-specific events occurring on a line, address, or call. The meaning of the message and the interpretation of the parameters are device specific.";
      asCIDMsg = "TAPI Detected an event: [LINE_DEVSPECIFIC] Param1: [" + IntToStr(iDwParam1) + "] Param2: [" + IntToStr(iDwParam2) + "] Text: [" + asMsg + "]";
      break;

    case LINE_DEVSPECIFICFEATURE:
      asMsg = "The LINE_DEVSPECIFICFEATURE message is sent to notify the application about device-specific events occurring on a line, address, or call. The meaning of the message and the interpretation of the parameters are device specific.";
      asCIDMsg = "TAPI Detected an event: [LINE_DEVSPECIFICFEATURE] Param1: [" + IntToStr(iDwParam1) + "] Param2: [" + IntToStr(iDwParam2) + "] Text: [" + asMsg + "]";
      break;

    case LINE_GATHERDIGITS:
      asMsg = "The LINE_GATHERDIGITS message is sent when the current buffered digit-gathering request has terminated or is canceled. The digit buffer may be examined after this message has been received by the application.";
      asCIDMsg = "TAPI Detected an event: [LINE_GATHERDIGITS] Param1: [" + IntToStr(iDwParam1) + "] Param2: [" + IntToStr(iDwParam2) + "] Text: [" + asMsg + "]";
      break;

    case LINE_GENERATE:
      asMsg = "The LINE_GENERATE message is sent to notify the application that the current digit or tone generation has terminated. Only one such generation request can be in progress an a given call at any time. This message is also sent when digit or tone generation is canceled.";
      asCIDMsg = "TAPI Detected an event: [LINE_GENERATE] Param1: [" + IntToStr(iDwParam1) + "] Param2: [" + IntToStr(iDwParam2) + "] Text: [" + asMsg + "]";
      break;

    case LINE_LINEDEVSTATE:
      switch(iDwParam1)
      {
        case LINEDEVSTATE_OTHER:
          asMsg = "Device-status items other than those listed below have changed. The application should check the current device status to determine which items have changed.";
          asCIDMsg = "TAPI Detected an event: [LINEDEVSTATE_OTHER] Param1: [" + IntToStr(iDwParam1) + "] Param2: [" + IntToStr(iDwParam2) + "] Text: [" + asMsg + "]";
          break;

        case LINEDEVSTATE_RINGING:
          asMsg = "The switch tells the line to alert the user. Service providers notify applications on each ring cycle by sending messages containing this constant. For example, in the United States, service providers send a message with this constant every six seconds.";
          asCIDMsg = "TAPI Detected an event: [LINEDEVSTATE_RINGING] Param1: [" + IntToStr(iDwParam1) + "] Param2: [" + IntToStr(iDwParam2) + "] Text: [" + asMsg + "]";
          break;

        case LINEDEVSTATE_CONNECTED:
          asMsg = "The line was previously disconnected and is now connected to TAPI.";
          asCIDMsg = "TAPI Detected an event: [LINEDEVSTATE_CONNECTED] Param1: [" + IntToStr(iDwParam1) + "] Param2: [" + IntToStr(iDwParam2) + "] Text: [" + asMsg + "]";
          break;

        case LINEDEVSTATE_DISCONNECTED:
          asMsg = "This line was previously connected and is now disconnected from TAPI.";
          asCIDMsg = "TAPI Detected an event: [LINEDEVSTATE_DISCONNECTED] Param1: [" + IntToStr(iDwParam1) + "] Param2: [" + IntToStr(iDwParam2) + "] Text: [" + asMsg + "]";
          break;

        case LINEDEVSTATE_MSGWAITON:
          asMsg = "The 'message waiting' indicator is turned on.";
          asCIDMsg = "TAPI Detected an event: [LINEDEVSTATE_MSGWAITON] Param1: [" + IntToStr(iDwParam1) + "] Param2: [" + IntToStr(iDwParam2) + "] Text: [" + asMsg + "]";
          break;

        case LINEDEVSTATE_MSGWAITOFF:
          asMsg = "The 'message waiting' indicator is turned off.";
          asCIDMsg = "TAPI Detected an event: [LINEDEVSTATE_MSGWAITOFF] Param1: [" + IntToStr(iDwParam1) + "] Param2: [" + IntToStr(iDwParam2) + "] Text: [" + asMsg + "]";
          break;

        case LINEDEVSTATE_NUMCOMPLETIONS:
          asMsg = "The number of outstanding call completions on the line device has changed.";
          asCIDMsg = "TAPI Detected an event: [LINEDEVSTATE_NUMCOMPLETIONS] Param1: [" + IntToStr(iDwParam1) + "] Param2: [" + IntToStr(iDwParam2) + "] Text: [" + asMsg + "]";
          break;

        case LINEDEVSTATE_INSERVICE:
          asMsg = "The line is connected to TAPI. This happens when TAPI is first activated, or when the line wire is physically plugged in and in service at the switch while TAPI is active.";
          asCIDMsg = "TAPI Detected an event: [LINEDEVSTATE_INSERVICE] Param1: [" + IntToStr(iDwParam1) + "] Param2: [" + IntToStr(iDwParam2) + "] Text: [" + asMsg + "]";
          break;

        case LINEDEVSTATE_OUTOFSERVICE:
          asMsg = "The line is out of service at the switch or physically disconnected. TAPI cannot be used to operate on the line device.";
          asCIDMsg = "TAPI Detected an event: [LINEDEVSTATE_OUTOFSERVICE] Param1: [" + IntToStr(iDwParam1) + "] Param2: [" + IntToStr(iDwParam2) + "] Text: [" + asMsg + "]";
          break;

        case LINEDEVSTATE_MAINTENANCE:
          asMsg = "Maintenance is being performed on the line at the switch. TAPI cannot be used to operate on the line device.";
          asCIDMsg = "TAPI Detected an event: [LINEDEVSTATE_MAINTENANCE] Param1: [" + IntToStr(iDwParam1) + "] Param2: [" + IntToStr(iDwParam2) + "] Text: [" + asMsg + "]";
          break;

        case LINEDEVSTATE_OPEN:
          asMsg = "The line has been opened by another application.";
          asCIDMsg = "TAPI Detected an event: [LINEDEVSTATE_OPEN] Param1: [" + IntToStr(iDwParam1) + "] Param2: [" + IntToStr(iDwParam2) + "] Text: [" + asMsg + "]";
          break;

        case LINEDEVSTATE_CLOSE:
          asMsg = "The line has been closed by another application.";
          asCIDMsg = "TAPI Detected an event: [LINEDEVSTATE_CLOSE] Param1: [" + IntToStr(iDwParam1) + "] Param2: [" + IntToStr(iDwParam2) + "] Text: [" + asMsg + "]";
          break;

        case LINEDEVSTATE_NUMCALLS:
          asMsg = "The number of calls on the line device has changed.";
          asCIDMsg = "TAPI Detected an event: [LINEDEVSTATE_NUMCALLS] Param1: [" + IntToStr(iDwParam1) + "] Param2: [" + IntToStr(iDwParam2) + "] Text: [" + asMsg + "]";
          break;

        case LINEDEVSTATE_TERMINALS:
          asMsg = "The terminal settings have changed.";
          asCIDMsg = "TAPI Detected an event: [LINEDEVSTATE_TERMINALS] Param1: [" + IntToStr(iDwParam1) + "] Param2: [" + IntToStr(iDwParam2) + "] Text: [" + asMsg + "]";
          break;

        case LINEDEVSTATE_ROAMMODE:
          asMsg = "The roaming state of the line device has changed.";
          asCIDMsg = "TAPI Detected an event: [LINEDEVSTATE_ROAMMODE] Param1: [" + IntToStr(iDwParam1) + "] Param2: [" + IntToStr(iDwParam2) + "] Text: [" + asMsg + "]";
          break;

        case LINEDEVSTATE_BATTERY:
          asMsg = "The battery level has changed significantly (cellular).";
          asCIDMsg = "TAPI Detected an event: [LINEDEVSTATE_BATTERY] Param1: [" + IntToStr(iDwParam1) + "] Param2: [" + IntToStr(iDwParam2) + "] Text: [" + asMsg + "]";
          break;

        case LINEDEVSTATE_SIGNAL:
          asMsg = "The signal level has changed significantly (cellular).";
          asCIDMsg = "TAPI Detected an event: [LINEDEVSTATE_SIGNAL] Param1: [" + IntToStr(iDwParam1) + "] Param2: [" + IntToStr(iDwParam2) + "] Text: [" + asMsg + "]";
          break;

        case LINEDEVSTATE_DEVSPECIFIC:
          asMsg = "The line's device-specific information has changed.";
          asCIDMsg = "TAPI Detected an event: [LINEDEVSTATE_DEVSPECIFIC] Param1: [" + IntToStr(iDwParam1) + "] Param2: [" + IntToStr(iDwParam2) + "] Text: [" + asMsg + "]";
          break;

        case LINEDEVSTATE_REINIT:
          asMsg = "Items have changed in the configuration of line devices. To become aware of these changes (as with the appearance of new line devices), the application should reinitialize its use of TAPI. The dwDevice parameter is left NULL for this state change as it applies to any of the lines in the system.";
          asCIDMsg = "TAPI Detected an event: [LINEDEVSTATE_REINIT] Param1: [" + IntToStr(iDwParam1) + "] Param2: [" + IntToStr(iDwParam2) + "] Text: [" + asMsg + "]";
          break;

        case LINEDEVSTATE_LOCK:
          asMsg = "The locked status of the line device has changed. (For more information, refer to the description of the LINEDEVSTATUSFLAGS_LOCKED bit of the LINEDEVSTATUSFLAGS_ constants.)";
          asCIDMsg = "TAPI Detected an event: [LINEDEVSTATE_LOCK] Param1: [" + IntToStr(iDwParam1) + "] Param2: [" + IntToStr(iDwParam2) + "] Text: [" + asMsg + "]";
          break;

        default:
          asMsg = "UNKNOWN LINEDEVSTATE TAPI EVENT";
          asCIDMsg = "TAPI Detected an event with ID: [" + IntToStr(iMsgID) + "] Param1: [" + IntToStr(iDwParam1) + "] Text: [" + asMsg + "]";
          break;
      }

      break;

    case LINE_MONITORDIGITS:
      asMsg = "The LINE_MONITORDIGITS message is sent when a digit is detected. The sending of this message is controlled with the lineMonitorDigits function.";
      asCIDMsg = "TAPI Detected an event: [LINE_MONITORDIGITS] Param1: [" + IntToStr(iDwParam1) + "] Param2: [" + IntToStr(iDwParam2) + "] Text: [" + asMsg + "]";
      break;

    case LINE_MONITORMEDIA:
      asMsg = "The LINE_MONITORMEDIA message is sent when a change in the call's media mode is detected. The sending of this message is controlled with the lineMonitorMedia function.";
      asCIDMsg = "TAPI Detected an event: [LINE_MONITORMEDIA] Param1: [" + IntToStr(iDwParam1) + "] Param2: [" + IntToStr(iDwParam2) + "] Text: [" + asMsg + "]";
      break;

    case LINE_MONITORTONE:
      asMsg = "The LINE_MONITORTONE message is sent when a tone is detected. The sending of this message is controlled with the lineMonitorTones function.";
      asCIDMsg = "TAPI Detected an event: [LINE_MONITORTONE] Param1: [" + IntToStr(iDwParam1) + "] Param2: [" + IntToStr(iDwParam2) + "] Text: [" + asMsg + "]";
      break;

    case LINE_REPLY:
      asMsg = "The LINE_REPLY message is sent to report the results of function calls that completed asynchronously.";
      asCIDMsg = "TAPI Detected an event: [LINE_REPLY] Param1: [" + IntToStr(iDwParam1) + "] Param2: [" + IntToStr(iDwParam2) + "] Text: [" + asMsg + "]";
      break;

    case LINE_REQUEST:
      asMsg = "The LINE_REQUEST message is sent to report the arrival of a new request from another application.";
      asCIDMsg = "TAPI Detected an event: [LINE_REQUEST] Param1: [" + IntToStr(iDwParam1) + "] Param2: [" + IntToStr(iDwParam2) + "] Text: [" + asMsg + "]";
      break;

    case PHONE_BUTTON:
      asMsg = "The PHONE_BUTTON message is sent to notify the application that button press monitoring is enabled if it has detected a button press on the local phone.";
      asCIDMsg = "TAPI Detected an event: [PHONE_BUTTON] Param1: [" + IntToStr(iDwParam1) + "] Param2: [" + IntToStr(iDwParam2) + "] Text: [" + asMsg + "]";
      break;

    case PHONE_CLOSE:
      asMsg = "The PHONE_CLOSE message is sent when an open phone device has been forcibly closed as part of resource reclamation. The device handle is no longer valid once this message has been sent.";
      asCIDMsg = "TAPI Detected an event: [PHONE_CLOSE] Param1: [" + IntToStr(iDwParam1) + "] Param2: [" + IntToStr(iDwParam2) + "] Text: [" + asMsg + "]";
      break;

    case PHONE_DEVSPECIFIC:
      asMsg = "The implementation sends the PHONE_DEVSPECIFIC message to an application to notify the application about device-specific events occurring at the phone. The meaning of the message and the interpretation of the parameters is implementation-defined.";
      asCIDMsg = "TAPI Detected an event: [PHONE_DEVSPECIFIC] Param1: [" + IntToStr(iDwParam1) + "] Param2: [" + IntToStr(iDwParam2) + "] Text: [" + asMsg + "]";
      break;

    case PHONE_REPLY:
      asMsg = "The PHONE_REPLY message is sent to an application' to report the results of function call that completed asynchronously.";
      asCIDMsg = "TAPI Detected an event: [PHONE_REPLY] Param1: [" + IntToStr(iDwParam1) + "] Param2: [" + IntToStr(iDwParam2) + "] Text: [" + asMsg + "]";
      break;

    case PHONE_STATE:
      asMsg = "TAPI sends the PHONE_STATE message to an application whenever the status of a phone device changes.";
      asCIDMsg = "TAPI Detected an event: [PHONE_STATE] Param1: [" + IntToStr(iDwParam1) + "] Param2: [" + IntToStr(iDwParam2) + "] Text: [" + asMsg + "]";
      break;

    case LINE_CREATE:
      asMsg = "The LINE_CREATE message is sent to inform the application of the creation of a new line device.";
      asCIDMsg = "TAPI Detected an event: [LINE_CREATE] Param1: [" + IntToStr(iDwParam1) + "] Param2: [" + IntToStr(iDwParam2) + "] Text: [" + asMsg + "]";
      break;

    case PHONE_CREATE:
      asMsg = "The PHONE_CREATE message is sent to inform applications of the creation of a new phone device.";
      asCIDMsg = "TAPI Detected an event: [PHONE_CREATE] Param1: [" + IntToStr(iDwParam1) + "] Param2: [" + IntToStr(iDwParam2) + "] Text: [" + asMsg + "]";
      break;

#if (TAPI_CURRENT_VERSION >= 0x00020000)
    case LINE_AGENTSPECIFIC:
      asMsg = "The LINE_AGENTSPECIFIC message is sent when the status of an ACD agent changes on a line the application currently has open. The application can invoke lineGetAgentStatus to determine the current status of the agent.";
      asCIDMsg = "TAPI Detected an event: [LINE_AGENTSPECIFIC] Param1: [" + IntToStr(iDwParam1) + "] Param2: [" + IntToStr(iDwParam2) + "] Text: [" + asMsg + "]";
      break;

    case LINE_AGENTSTATUS:
      asMsg = "The LINE_AGENTSTATUS message is sent when the status of an ACD agent changes on a line the application currently has open. The application can invoke lineGetAgentStatus to determine the current status of the agent.";
      asCIDMsg = "TAPI Detected an event: [LINE_AGENTSTATUS] Param1: [" + IntToStr(iDwParam1) + "] Param2: [" + IntToStr(iDwParam2) + "] Text: [" + asMsg + "]";
      break;

    case LINE_APPNEWCALL:
      asMsg = "The LINE_APPNEWCALL message is sent to inform an application when a new call handle has been spontaneously created on its behalf (other than through an API call from the application, in which case the handle would have been returned through a pointer parameter passed into the function).";
      asCIDMsg = "TAPI Detected an event: [LINE_APPNEWCALL] Param1: [" + IntToStr(iDwParam1) + "] Param2: [" + IntToStr(iDwParam2) + "] Text: [" + asMsg + "]";
      break;

    case LINE_PROXYREQUEST:
      asMsg = "The LINE_PROXYREQUEST message delivers a request to a registered proxy function handler.";
      asCIDMsg = "TAPI Detected an event: [LINE_PROXYREQUEST] Param1: [" + IntToStr(iDwParam1) + "] Param2: [" + IntToStr(iDwParam2) + "] Text: [" + asMsg + "]";
      break;

    case LINE_REMOVE:
      asMsg = "The LINE_REMOVE message is sent to inform an application of the removal (deletion from the system) of a line device. Generally, this is not used for temporary removals, such as extraction of PCMCIA devices, but only for permanent removals in which the device would no longer be reported by the service provider if TAPI were reinitialized.";
      asCIDMsg = "TAPI Detected an event: [LINE_REMOVE] Param1: [" + IntToStr(iDwParam1) + "] Param2: [" + IntToStr(iDwParam2) + "] Text: [" + asMsg + "]";
      break;

    case PHONE_REMOVE:
      asMsg = "he PHONE_REMOVE message is sent to inform an application of the removal (deletion from the system) of a phone device. Generally, this is not used for temporary removals, such as extraction of PCMCIA devices, but only for permanent removals in which the device would no longer be reported by the service provider if TAPI were reinitialized.";
      asCIDMsg = "TAPI Detected an event: [PHONE_REMOVE] Param1: [" + IntToStr(iDwParam1) + "] Param2: [" + IntToStr(iDwParam2) + "] Text: [" + asMsg + "]";
      break;
#endif

#if (TAPI_CURRENT_VERSION >= 0x00020002)
    case LINE_AGENTSESSIONSTATUS:
      asMsg = "The LINE_AGENTSESSIONSTATUS message ???.";
      asCIDMsg = "TAPI Detected an event: [LINE_AGENTSESSIONSTATUS] Param1: [" + IntToStr(iDwParam1) + "] Param2: [" + IntToStr(iDwParam2) + "] Text: [" + asMsg + "]";
      break;

    case LINE_QUEUESTATUS:
      asMsg = "The LINE_QUEUESTATUS message ???.";
      asCIDMsg = "TAPI Detected an event: [LINE_QUEUESTATUS] Param1: [" + IntToStr(iDwParam1) + "] Param2: [" + IntToStr(iDwParam2) + "] Text: [" + asMsg + "]";
      break;

    case LINE_AGENTSTATUSEX:
      asMsg = "The LINE_AGENTSTATUSEX message ???.";
      asCIDMsg = "TAPI Detected an event: [LINE_AGENTSTATUSEX] Param1: [" + IntToStr(iDwParam1) + "] Param2: [" + IntToStr(iDwParam2) + "] Text: [" + asMsg + "]";
      break;

    case LINE_GROUPSTATUS:
      asMsg = "The LINE_GROUPSTATUS message ???.";
      asCIDMsg = "TAPI Detected an event: [LINE_GROUPSTATUS] Param1: [" + IntToStr(iDwParam1) + "] Param2: [" + IntToStr(iDwParam2) + "] Text: [" + asMsg + "]";
      break;

    case LINE_PROXYSTATUS:
      asMsg = "The LINE_PROXYSTATUS message ???.";
      asCIDMsg = "TAPI Detected an event: [LINE_PROXYSTATUS] Param1: [" + IntToStr(iDwParam1) + "] Param2: [" + IntToStr(iDwParam2) + "] Text: [" + asMsg + "]";
      break;
#endif

#if (TAPI_CURRENT_VERSION >= 0x00030000)
    case LINE_APPNEWCALLHUB:
      asMsg = "The LINE_APPNEWCALLHUB message ???.";
      asCIDMsg = "TAPI Detected an event: [LINE_APPNEWCALLHUB] Param1: [" + IntToStr(iDwParam1) + "] Param2: [" + IntToStr(iDwParam2) + "] Text: [" + asMsg + "]";
      break;

    case LINE_CALLHUBCLOSE:
      asMsg = "The LINE_CALLHUBCLOSE message ???.";
      asCIDMsg = "TAPI Detected an event: [LINE_CALLHUBCLOSE] Param1: [" + IntToStr(iDwParam1) + "] Param2: [" + IntToStr(iDwParam2) + "] Text: [" + asMsg + "]";
      break;

    case LINE_DEVSPECIFICEX:
      asMsg = "The LINE_DEVSPECIFICEX message ???.";
      asCIDMsg = "TAPI Detected an event: [LINE_DEVSPECIFICEX] Param1: [" + IntToStr(iDwParam1) + "] Param2: [" + IntToStr(iDwParam2) + "] Text: [" + asMsg + "]";
      break;
#endif

    default:
      asMsg = "UNKNOWN TAPI EVENT";
      asCIDMsg = "TAPI Detected an event with ID: [" + IntToStr(iMsgID) + "] Param1: [" + IntToStr(iDwParam1) + "] Text: [" + asMsg + "]";
      break;
  }

  memoTAPI->Lines->Add(asCIDMsg);

}
//---------------------------------------------------------------------------
void __fastcall TForm1::vsTAPICallerIDEvent(AnsiString asName,
      AnsiString asNumber, int iCallType, HCALL hCall)
{
  hIncomingCall_m = hCall;

  memoTAPI->Lines->Add("TAPI Detected CallerID event: Name: " + asName + " Phone: " + asNumber + " Call Type: " + IntToStr(iCallType));
}
//---------------------------------------------------------------------------
void __fastcall TForm1::bbtnPlaySoundTAPIClick(TObject *Sender)
{
  if(vsTAPI->AutomatedVoiceAllowed == true)
  {
    AnsiString asErr = "";
    if(vsTAPI->PlayASound(edtTestWav->Text, &asErr) == false)
    {
      memoTAPI->Lines->Add(asErr);
      ShowMessage(asErr);
    }

    //CloseCall(hIncomingCall_m);
  }

}
//---------------------------------------------------------------------------
void __fastcall TForm1::bbtnAnswerTAPIClick(TObject *Sender)
{
  if(vsTAPI->AutomatedVoiceAllowed == true)
  {
    if(vsTAPI->AnswerTheLine(hIncomingCall_m) == true)
    {
      memoTAPI->Lines->Add("Call Answered for hCall Handle: " + hIncomingCall_m);
    }
    else
    {
      memoTAPI->Lines->Add("FAILED TO ANSWER Call for hCall Handle: " + hIncomingCall_m);
    }
  }

}
//---------------------------------------------------------------------------
void __fastcall TForm1::bbtnHangupTAPIClick(TObject *Sender)
{
  vsTAPI->CloseCall(hIncomingCall_m);
}

//---------------------------------------------------------------------------
void __fastcall TForm1::bbtnRecordTAPIClick(TObject *Sender)
{
  if(vsTAPI->AutomatedVoiceAllowed == true)
  {
    AnsiString asErr = "";
    if(vsTAPI->RecordASound(edtTestWav->Text, hIncomingCall_m, edtMaxRecordTime->Text.ToInt(),&asErr) == false)
    {
      memoTAPI->Lines->Add(asErr);
      //ShowMessage(asErr);
    }
    else
    {
      memoTAPI->Lines->Add("DONE Recording!");
    }
  }

}
//---------------------------------------------------------------------------
void __fastcall TForm1::bbtnCallClick(TObject *Sender)
{
  AnsiString asName = "Test Call";
  bool bBlockCallerID = false;

  if(InputQuery("Enter Phone # to call","Phone #:",asCallPhoneNum_m) == false)
  {
    return;
  }

  vsTAPI->MakeACall(asName, asCallPhoneNum_m, 0, bBlockCallerID, 0 &hIncomingCall_m);
}
//---------------------------------------------------------------------------
void __fastcall TForm1::FormCloseQuery(TObject *Sender, bool &CanClose)
{
  WriteToRegistry("", HKEY_CURRENT_USER, "SOFTWARE\\VSOFT\\TAPITest", "WavFile" , edtTestWav->Text);
  WriteToRegistry("", HKEY_CURRENT_USER, "SOFTWARE\\VSOFT\\TAPITest", "CallPhoneNumber" , asCallPhoneNum_m);
  WriteToRegistry("", HKEY_CURRENT_USER, "SOFTWARE\\VSOFT\\TAPITest", "MaxRecordSecs" , edtMaxRecordTime->Text.ToIntDef(300));

}
//---------------------------------------------------------------------------
void __fastcall TForm1::vsTAPICallConnectedEvent(AnsiString asAppName,
      DWORD dwBearerMode, DWORD dwMediaMode, DWORD dwRate,
      AnsiString asCalledPartyName, AnsiString asComments, HCALL hCall)
{
  memoTAPI->Lines->Add("TAPI component called CallConnected Event.");
  hIncomingCall_m = hCall;

}
//---------------------------------------------------------------------------

void __fastcall TForm1::vsTAPICallDisconnectedEvent(HCALL hCall)
{
  memoTAPI->Lines->Add("TAPI component called CallDisconnected Event.");
  hIncomingCall_m = hCall;

  vsTAPI->StopRecording();
}
//---------------------------------------------------------------------------

void __fastcall TForm1::bbtnStopRecordTAPIClick(TObject *Sender)
{
  vsTAPI->StopRecording();  
}

//---------------------------------------------------------------------------
void __fastcall TForm1::vsTAPICallRingEvent(int iRingCount,
      const HCALL &hCall)
{
  memoTAPI->Lines->Add("Detected RING event via TAPI, RINGS: " + IntToStr(iRingCount));
}

//---------------------------------------------------------------------------

